from typing import Any, NamedTuple


class ResolverInputSet(NamedTuple):
    parent: Any = ...
    driver: Any = ...
    locator: Any = ...
    full_locator: str = ...
    strategy: str = ...
    logger: Any = ...
